dg_Grid

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Returns a dg_DistanceGrid with StartX,StartY as its center.  If
		//	ExitsNotEntrances, the distances are based on exits.  Otherwise,
		//	they're based on entrances.  Returns null on failure.
		//
		//	By default, this records the distance to every node accessible from
		//	StartX,StartY.  If EndX and EndY are not null, the table will stop
		//	recording distances when/if it is reached.  Remember to check the
		//	table's dg_GetEndX() and dg_GetEndY() procs to verify that
		//	EndX,EndY was actually accessible.
		dg_GetDistanceGrid(StartX,StartY,ExitsNotEntrances=TRUE,EndX=null,EndY=null)
			if ( !src.grid2d_HasCoordinates(StartX,StartY) )	return null
			if ( (EndX!=null) && (EndY!=null) && !src.grid2d_HasCoordinates(EndX,EndY) )
				return null
			return new/dg_DistanceGrid(src,StartX,StartY,ExitsNotEntrances,EndX,EndY)

////////////////////////////////////////////////////////////////////////////////
dg_DistanceGrid
	parent_type	=/grid2d_Grid

	//==========================================================================
	var
		dg_isExits	= TRUE
		//	Indices of the first an dlast nodes checked.
		dg_start	= 0
		dg_end		= 0

	//==========================================================================
	//	PRIVATE
	New(dg_Grid/Maze,StartX,StartY,ExitsNotEntrances=TRUE,EndX=null,EndY=null)
		//	Initialize dimensions.  Default distance is -1.
		if ( !istype(Maze,/dg_Grid) )	return ..(1,1,-1)
		..(Maze.grid2d_GetMaxX(),Maze.grid2d_GetMaxY(),-1)
		//	Privacy makes this boolean check extra dumb, but whatever. ;)
		if (ExitsNotEntrances)			src.dg_isExits	= TRUE
		else							src.dg_isExits	= FALSE
		src.dg_start					= src.grid2d_XYToIndex(StartX,StartY)
		src.dg_end						= src.grid2d_XYToIndex(EndX,EndY)
		if (!src.dg_start)				return
		src.grid2d_cells[src.dg_start]	= 0

		var
			distance	= 0
			list/coords
			index
			start	= 1
			end		= 1
			storage

		if (src.dg_isExits)
			coords	= Maze.dg_GetExits(StartX,StartY)//	EXITS
			end		= coords.len/2
			while(start<=end)
				distance++
				for(index = start to end)
					//	Set the distance of unset coordinates.
					storage	= src.grid2d_XYToIndex(coords[2*index-1],coords[2*index])
					if (!storage || (src.grid2d_cells[storage]>=0))	continue
					src.grid2d_cells[storage]						= distance
					//	Set an early ending?
					if ( (coords[2*index-1]==EndX) && (coords[2*index]==EndY) )
						coords.Cut(end+1,0)
						break
					//	Gather new exit coordinates.
					coords	+= Maze.dg_GetExits(coords[2*index-1],coords[2*index])//	EXITS
				//	Prepare to test the new coordinates.
				start	= end+1
				end		= coords.len/2
		else
			coords	= Maze.dg_GetEntrances(StartX,StartY)//	ENTRANCES
			end		= coords.len/2
			while(start<=end)
				distance++
				for(index = start to end)
					//	Set the distance of unset coordinates.
					storage	= src.grid2d_XYToIndex(coords[2*index-1],coords[2*index])
					if (!storage || (src.grid2d_cells[storage]>=0))	continue
					src.grid2d_cells[storage]						= distance
					//	Set an early ending?
					if ( (coords[2*index-1]==EndX) && (coords[2*index]==EndY) )
						coords.Cut(end+1,0)
						break
					//	Gather new exit coordinates.
					coords	+= Maze.dg_GetEntrances(coords[2*index-1],coords[2*index])//	ENTRANCES
				//	Prepare to test the new coordinates.
				start	= end+1
				end		= coords.len/2


	//==========================================================================
	proc
		//----------------------------------------------------------------------
		dg_GetEndIndex()
			return src.dg_end
		//----------------------------------------------------------------------
		//	Returns the X coordinate of the last cell checked...  if defined!
		dg_GetEndX()
			return src.grid2d_IndexToX(src.dg_end)
		//----------------------------------------------------------------------
		//	Returns the Y coordinate of the last cell checked...  if defined!
		dg_GetEndY()
			return src.grid2d_IndexToY(src.dg_end)

		//----------------------------------------------------------------------
		//	Return sthe maximum distance stored in the table.
		dg_GetMaxDistance()
			return max(src.grid2d_cells)

		//----------------------------------------------------------------------
		dg_GetStartIndex()
			return src.dg_start
		//----------------------------------------------------------------------
		//	Returns the X coordinate of the cell the distances are from.
		dg_GetStartX()
			return src.grid2d_IndexToX(src.dg_start)
		//----------------------------------------------------------------------
		//	Returns the Y coordinate of the cell the distances are from.
		dg_GetStartY()
			return src.grid2d_IndexToY(src.dg_start)

		//----------------------------------------------------------------------
		//	If true, this is storing exit distances.  Otherwise, it's storing
		//	entrance distances.
		dg_IsExits()
			return src.dg_isExits
