dg_Grid

/*

	Note that it is often wise to deal with indices rather than coordinates
	so that cells can be identified by a single key.

*/

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Draws a maze within the grid using Prim's algorithm.  If StartX and
		//	StartY are not valid coordinates, a random start cell will be chosen.
		//	If ReciprocateExits, connections between cells go both ways.
		//
		//	Note that previous grid information will not be cleared first.
		//	If desired, that should be done manually with grid2d_Reset().
		dg_Prim(StartX,StartY,ReciprocateExits=FALSE)
			//	Make sure the start node is one that is included.
			var/list/nodes	= src.dg_GetIncludedIndices()
			.				= src.grid2d_XYToIndex(StartX,StartY)
			if ( !(.) || src.dg_IsExcluded(StartX,StartY) )
				.			= nodes[rand(1,nodes.len)]
			//	Find "frontier" neighbors of the starting node.
			var
				list/nodesIn		= list(.)
				list/nodesFrontier	= src.grid2d_GetNeighborIndices(.)
				nodeFrontier
				list/neighborsIn	= list()
				neighbor
				neighborIn
			//	While there are nodes on the "frontier"...
			while(nodesFrontier.len)
				//	Pick one.
				nodeFrontier	= nodesFrontier[rand(1,nodesFrontier.len)]
				nodesFrontier	-=	nodeFrontier
				//	Find that node's neighbors.
				neighborsIn.Cut()
				for( neighbor in src.grid2d_GetNeighborIndices(nodeFrontier) )
					//	Collect the "in" neighbors.
					if ( nodesIn.Find(neighbor) )				neighborsIn		+= neighbor
					//	Collect "out" neighbors for the frontier.
					else if ( !nodesFrontier.Find(neighbor) )	nodesFrontier	+= neighbor
				//	Link a random "in" neighbor to the selected "frontier" node.
				neighborIn	= neighborsIn[rand(1,neighborsIn.len)]
				src.dg_SetExitCell( \
					TRUE, \
					src.grid2d_IndexToX(neighborIn), src.grid2d_IndexToY(neighborIn), \
					src.grid2d_IndexToX(nodeFrontier), src.grid2d_IndexToY(nodeFrontier), \
					)
				if (ReciprocateExits)
					src.dg_SetExitCell( \
						TRUE, \
						src.grid2d_IndexToX(nodeFrontier), src.grid2d_IndexToY(nodeFrontier), \
						src.grid2d_IndexToX(neighborIn), src.grid2d_IndexToY(neighborIn), \
						)
				//	Mark the "frontier" node as "in".
				nodesIn	+= nodeFrontier
