dg_Grid


	//==========================================================================
	var/list/dg_exclusions

	//==========================================================================
	//
	//	Redefine neighbors to ignore exclusions.
	//

	//--------------------------------------------------------------------------
	grid2d_AreNeighbors(X1,Y1,X2,Y2)
		if (!src.dg_exclusions)	return ..(X1,Y1,X2,Y2)
		if ( src.dg_IsExcluded(X1,Y1) || src.dg_IsExcluded(X2,Y2) )	return FALSE
		return ..(X1,Y1,X2,Y2)

	//--------------------------------------------------------------------------
	grid2d_GetNeighbors(X,Y)
		if (!src.dg_exclusions)	return ..(X,Y)
		var
			list/results		= ..(X,Y)
			index				= length(results)
		while(index>0)
			if ( src.dg_IsExcluded(results[index-1],results[index]) )
				results.Cut(index-1,0)
			index				-= 2
		return results

	//--------------------------------------------------------------------------
	grid2d_GetNeighborIndices(Index)
		if (!src.dg_exclusions)	return ..(Index)
		.						= ..(Index) - src.dg_exclusions

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Adds the coordinates X,Y to the exclusion list.  Setting EnsureUnique
		//	to true will avoid the risk of leaving copies behind when removing,
		//	but it makes this slower with a large number of exclusions.
		//	Returns
		//		0 on failure
		//		-1 if it was already in there and EnsureUnique is true
		//		the grid index added (always positive)
		dg_AddExclusion(X,Y,EnsureUnique=FALSE)
			var/index	= src.grid2d_XYToIndex(X,Y)
			if (!index)	return 0
			if (EnsureUnique && (index in src.dg_exclusions))	return -1
			if (!src.dg_exclusions)			src.dg_exclusions	= list()
			src.dg_exclusions				+= index
			return index

		//----------------------------------------------------------------------
		//	Removes all exclusions and returns the list of indices formally excluded.
		dg_ClearExclusions()
			if (!src.dg_exclusions)	return list()
			.						= src.dg_exclusions
			src.dg_exclusions		= null

		//----------------------------------------------------------------------
		//	Returns a list containing the grid indices excluded.  If Copy is false
		//	this is the actual internal storage.  (Don't edit it directly!)
		dg_GetExcludedIndices(Copy=TRUE)
			if (!src.dg_exclusions)	return list()
			if (Copy)				return src.dg_exclusions.Copy()
			return src.dg_exclusions

		//----------------------------------------------------------------------
		//	Returns a list containing the coordinates excluded.
		dg_GetExcluded()
			if (!src.dg_exclusions)	return list()
			.	= list()
			for(var/index in src.dg_exclusions)
				.	+= src.grid2d_IndexToX(index)
				.	+= src.grid2d_IndexToY(index)

		//----------------------------------------------------------------------
		//	Returns a list containing the indices which are not excluded.
		dg_GetIncludedIndices()
			.						= src.grid2d_GetIndices()
			if (src.dg_exclusions)	.	-= src.dg_exclusions

		//----------------------------------------------------------------------
		//	Returns a list containing the coordinates which are not excluded.
		dg_GetIncluded()
			if (!src.dg_exclusions)	return src.grid2d_GetCoordinates()
			var
				X
				Y
				maxX	= src.grid2d_GetMaxX()
				maxY	= src.grid2d_GetMaxY()
			.				= list()
			for(Y = 1 to maxY)
				for(X = 1 to maxX)
					if ( !src.dg_IsExcluded(X,Y) )
						.	+= X
						.	+= Y

		//----------------------------------------------------------------------
		//	Returns whether or not the coordinates X,Y are excluded from the maze.
		dg_IsExcluded(X,Y)
			if (!src.dg_exclusions)	return FALSE
			return (src.grid2d_XYToIndex(X,Y) in src.dg_exclusions)

		//----------------------------------------------------------------------
		//	Removes the coordinates X,Y from the exclusion list.  Setting RemoveAll
		//	to true will avoid leaving copies behind, but it's slower.
		//	Returns
		//		0 on failure
		//		the amount of removals if RemoveAll is true
		//		the grid index removed if RemoveAll is false
		dg_RemoveExclusion(X,Y,RemoveAll=FALSE)
			if (!src.dg_exclusions)	return 0
			var/index	= src.grid2d_XYToIndex(X,Y)
			if (!index)	return 0
			var/oldLen
			.												= 0
			if (RemoveAll)//	Remove every occurance.
				oldLen										= src.dg_exclusions.len
				src.dg_exclusions							-= index
				while(oldLen>src.dg_exclusions.len)
					.										+= 1
					oldLen									= src.dg_exclusions.len
					src.dg_exclusions						-= index
			else//	Remove the first occurance.
				oldLen										= src.dg_exclusions.len
				src.dg_exclusions							-= index
				if (oldLen>src.dg_exclusions.len)			.	= index
			//	Dispose of empty lists.
			if (!src.dg_exclusions.len)	src.dg_exclusions	= null
